@@ -21,7 +21,7 @@ use std::net::{Ipv4Addr, SocketAddrV4, TcpListener};
 use std::path::PathBuf;
 use std::process::{Child, Command, ExitStatus, Stdio};
 use std::time::Duration;
-use std::{env, thread};
+use std::{env, fmt, thread};
 use tempfile::TempDir;
 
 pub extern crate bitcoincore_rpc;
@@ -68,12 +68,32 @@ pub enum P2P {
 }
 
 /// All the possible error in this crate
-#[derive(Debug)]
 pub enum Error {
     /// Wrapper of io Error
     Io(std::io::Error),
     /// Wrapper of bitcoincore_rpc Error
     Rpc(bitcoincore_rpc::Error),
+    /// Returned when calling methods requiring a feature to be activated, but it's not
+    NoFeature,
+    /// Returned when calling methods requiring a env var to exist, but it's not
+    NoEnvVar,
+    /// Returned when calling methods requiring either a feature or env var, but both are absent
+    NeitherFeatureNorEnvVar,
+    /// Returned when calling methods requiring either a feature or anv var, but both are present
+    BothFeatureAndEnvVar,
+}
+
+impl fmt::Debug for Error {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Error::Io(e) => write!(f, "{:?}", e),
+            Error::Rpc(e) => write!(f, "{:?}", e),
+            Error::NoFeature => write!(f, "Called a method requiring a feature to be set, but it's not"),
+            Error::NoEnvVar => write!(f, "Called a method requiring env var `BITCOIND_EXE` to be set, but it's not"),
+            Error::NeitherFeatureNorEnvVar =>  write!(f, "Called a method requiring env var `BITCOIND_EXE` or a feature to be set, but neither are set"),
+            Error::BothFeatureAndEnvVar => write!(f, "Called a method requiring env var `BITCOIND_EXE` or a feature to be set, but both are set"),
+        }
+    }
 }
 
 const LOCAL_IP: Ipv4Addr = Ipv4Addr::new(127, 0, 0, 1);
@@ -302,15 +322,26 @@ impl From<bitcoincore_rpc::Error> for Error {
 }
 
 /// Provide the bitcoind executable path if a version feature has been specified
-pub fn downloaded_exe_path() -> Option<String> {
+pub fn downloaded_exe_path() -> Result<String, Error> {
     if versions::HAS_FEATURE {
-        Some(format!(
+        Ok(format!(
             "{}/bitcoin/bitcoin-{}/bin/bitcoind",
             env!("OUT_DIR"),
             versions::VERSION
         ))
     } else {
-        None
+        Err(Error::NoFeature)
+    }
+}
+
+/// Returns the daemon executable path if it's provided as a feature or as `BITCOIND_EXE` env var.
+/// Returns error if none or both are set
+pub fn exe_path() -> Result<String, Error> {
+    match (downloaded_exe_path(), std::env::var("BITCOIND_EXE")) {
+        (Ok(_), Ok(_)) => Err(Error::BothFeatureAndEnvVar),
+        (Ok(path), Err(_)) => Ok(path),
+        (Err(_), Ok(path)) => Ok(path),
+        (Err(_), Err(_)) => Err(Error::NeitherFeatureNorEnvVar),
     }
 }
 
@@ -318,10 +349,9 @@ pub fn downloaded_exe_path() -> Option<String> {
 mod test {
     use crate::bitcoincore_rpc::jsonrpc::serde_json::Value;
     use crate::bitcoincore_rpc::Client;
-    use crate::downloaded_exe_path;
+    use crate::exe_path;
     use crate::{get_available_port, BitcoinD, Conf, LOCAL_IP, P2P};
     use bitcoincore_rpc::RpcApi;
-    use std::env;
     use std::net::SocketAddrV4;
 
     #[test]
@@ -382,17 +412,17 @@ mod test {
     fn test_multi_p2p() {
         let mut conf_node1 = Conf::default();
         conf_node1.p2p = P2P::Yes;
-        let node1 = BitcoinD::with_conf(exe_path(), &conf_node1).unwrap();
+        let node1 = BitcoinD::with_conf(exe_path().unwrap(), &conf_node1).unwrap();
 
         // Create Node 2 connected Node 1
         let mut conf_node2 = Conf::default();
         conf_node2.p2p = node1.p2p_connect(true).unwrap();
-        let node2 = BitcoinD::with_conf(exe_path(), &conf_node2).unwrap();
+        let node2 = BitcoinD::with_conf(exe_path().unwrap(), &conf_node2).unwrap();
 
         // Create Node 3 Connected To Node
         let mut conf_node3 = Conf::default();
         conf_node3.p2p = node2.p2p_connect(false).unwrap();
-        let node3 = BitcoinD::with_conf(exe_path(), &conf_node3).unwrap();
+        let node3 = BitcoinD::with_conf(exe_path().unwrap(), &conf_node3).unwrap();
 
         // Get each nodes Peers
         let node1_peers = peers_connected(&node1.client);
@@ -466,18 +496,8 @@ mod test {
         result.len()
     }
 
-    fn exe_path() -> String {
-        if let Some(downloaded_exe_path) = downloaded_exe_path() {
-            downloaded_exe_path
-        } else {
-            env::var("BITCOIND_EXE").expect(
-                "when no version feature is specified, you must specify BITCOIND_EXE env var",
-            )
-        }
-    }
-
     fn init() -> String {
         let _ = env_logger::try_init();
-        exe_path()
+        exe_path().unwrap()
     }
 }
